==> lhcsmapi-1.3.229.tar.gz: lhcsmapi-1.3.229/lhcsmapi/dbsignal/post_mortem/PmDbRequest.py <==
import requests
import json
import warnings
import pandas as pd
import numpy as np
import functools
from lhcsmapi.Time import Time


class PmDbRequest(object):
    """PmDbRequest class encapsulates PM database queries in order to simplify the query creation as well as browsing of
    PM events for a given period.
    """

    _request_types = ["pmevent, pmdata", "pmdata/signal"]
    _keyword_to_value_type = {"system": str, "className": str, "source": str, "signal": str,
                              "fromTimestampInNanos": int, "timestampInNanos": int, "durationInNanos": int}

    @staticmethod
    def generate_request(request_type, is_header=False, pm_rest_api_path ="http://pm-api-pro/v2/", **kwargs):
        """**Generates a PostMortem request**

            Method generates a HTTP REST Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration.

            :param request_type: type of PM request (supported: pmdata, pmdata/signal, pmevent)
            :type request_type: string
            :param is_header: flag indicating whether (if True) or not (if False) PM header should be requested
            :type is_header: boolean
            :param kwargs: dictionary of (keyword: argument) pairs which parametrize a db API query
            :type kwargs: dictionary
            :return: string with a PM HTTP request

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.generateRequest("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
               >>> http://pm-api-pro/v2/pmdata/within/duration?source=*&system=QPS&className=DQAMCNMB_PMHSU&fromTimestampInNanos=1426201200000000000&durationInNanos=86400000000000

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.generateRequest("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, duration = eventDurationSeconds)
                >>> ValueError: Some keywords: {'duration'} not in dict_keys(['system', 'className', 'source', 'signal', 'fromTimestampInNanos', 'timestampInNanos', 'durationInNanos'])!

        """

        # check if keys are present
        key_difference = kwargs.keys() - PmDbRequest._keyword_to_value_type.keys()
        if len(key_difference) > 0:
            raise ValueError("Some keywords: {} not in {}!".format(key_difference, PmDbRequest._keyword_to_value_type.keys()))

        # check if value types are correct

        # for time setting check if it is either timestampInNanos and not (fromTimestampInNanos or durationInNanos)
        if "timestampInNanos" in kwargs and any(type in ["fromTimestampInNanos", "durationInNanos"] for type in kwargs):
            ValueError(
                "If timestampInNanos is present, then both fromTimestampInNanos and durationInNanos have to be absent and vice versa!")

        # concatenate keys and values for a query
        keywords_and_argument_concat = '&'.join(['%s=%s' % (key, value) for (key, value) in kwargs.items()])

        if not any(type in request_type for type in request_type):
            ValueError("PM request type {} not supported.".format(request_type))

        request = "{}{}".format(pm_rest_api_path, request_type)

        if is_header:
            request = request + "/header"

        if "durationInNanos" in kwargs:
            return "{}/within/duration?{}".format(request, keywords_and_argument_concat)
        else:
            return "{}?{}".format(request, keywords_and_argument_concat)

    @staticmethod
    @functools.lru_cache(maxsize=16, typed=False)
    def get_response(request_type, is_header=False, display_warning = False, pm_rest_api_path ="http://pm-api-pro/v2/", **kwargs):
        """**Generates a PM request**

            Method returns content of an REST HTTP Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration.

            :param request_type: type of PM request (supported: pmdata, pmdata/signal, pmevent)
            :type request_type: string
            :param is_header: flag indicating whether (if True) or not (if False) PM header should be requested
            :type is_header: boolean
            :param kwargs: dictionary of (keyword: argument) pairs which parametrize a db API query
            :type kwargs: dictionary
            :return: string with a PM HTTP request

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
               >>> {'content': [{'valueDescriptor': {'attributes': {'QUENCHTIME': {'description': {'signal': 'description',
               >>> ...

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, duration = eventDurationSeconds)
                >>> ValueError: Some keywords: {'duration'} not in dict_keys(['system', 'className', 'source', 'signal', 'fromTimestampInNanos', 'timestampInNanos', 'durationInNanos'])!

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400
                PmDbRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
                >>> UserWarning: Querying Post Mortem failed using the following query:
                                http://pm-api-pro/v2/pmdata/within/duration?source=*&system=QPS&className=DQAMCNMB_PMHSU&fromTimestampInNanos=1426201200000000001&durationInNanos=86400

        """

        request = PmDbRequest.generate_request(request_type, is_header, pm_rest_api_path, **kwargs)
        response = requests.get(request)

        try:
            return json.loads(response.text)
        except:
            warning = """Querying Post Mortem failed using the following query: {}""".format(request)
            if display_warning:
                warnings.warn(warning)
            return ''

    @staticmethod
    def find_events(source, system, className, t_start, t_end=None, duration=None,
                    pm_rest_api_path="http://pm-api-pro/v2/", display_warning=False):
        """**Finds PM events for a given source, system, className as well as start and end time**

            Method returns content of an REST HTTP Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration. Three operation modes are possible:

            - if endTimeData is not None:

            (t_start, t_end) = (startTimeData, endTimeData)

            - elif len(durationData) is 1:

            (t_start, t_end) = (startTimeData, startTimeData+durationData[0])

            - elif len(durationData) is 2:

            (t_start, t_end) = (startTimeData-durationData[0], startTimeData+durationData[1])

            :param source: PM event source
            :type source: string
            :param system: PM event system
            :type system: string
            :param className: PM event class signal
            :type className: string
            :param t_start: input date and time
            :type t_start: int unix timestamp
            :param t_end: unit of the input (None if duration defined)
            :type t_end: int unix timestamp
            :param duration: time zone (None if t_end defined)
            :type duration: list of (time, unit) tuples
            :return: list of (source, timestamp) pairs

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 24*60*60

                PmDbRequest.findEvents(source, system, className,
                    eventTime = selectedDayTimeStamp, duration = [(eventDurationSeconds, 's')])
                >>> [('B20L5', 1426220469491000000),
                >>> ...

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 1
                PmDbRequest.findEvents(source, system, className,
                    eventTime = selectedDayTimeStamp, duration = [(eventDurationSeconds, 's')])
                >>> UserWarning: Querying Post Mortem failed using the following query:
                                http://pm-api-pro/v2/pmdata/header/within/duration?system=QPS&className=DQAMCNMB_PMHSU&source=*&fromTimestampInNanos=1426201200000000000&durationInNanos=1000000000 returned no data so an empty string is being returned

        """

        from_timestamp_in_nanos, end_time_in_nanos = Time.get_query_period_in_unix_time(t_start, t_end, duration)
        MAX_DURATION = int(24 * 60 * 60 * 1e9)
        duration_in_nanos = end_time_in_nanos - from_timestamp_in_nanos
        duration_in_nanos = min(duration_in_nanos, MAX_DURATION)

        # Fetch PM event data
        pm_headers = PmDbRequest.get_response("pmdata", True, display_warning, pm_rest_api_path, system=system,
                                              className=className, source=source,
                                              fromTimestampInNanos=from_timestamp_in_nanos, durationInNanos=duration_in_nanos)

        if "v2" in pm_rest_api_path:
            SOURCE_KEYWORD = "source"
        elif "v3" in pm_rest_api_path:
            SOURCE_KEYWORD = "sourceName"

        event_list = []
        for pm_header in pm_headers:
            if 'DT' not in pm_header[SOURCE_KEYWORD]:
                event_list.append((pm_header[SOURCE_KEYWORD], pm_header['timestamp']))

            # if 'DT' in 'source', then rubbish data injected into official PM data

        return event_list

    @staticmethod
    def find_events_between_dates(t_start, t_end, className, source, system):
        quotient, remainder = Time.modulo_day_in_unix_timestamp(t_start, t_end)
        event_duration_nano_seconds = int(24 * 60 * 60 * 1e9)
        event_source_and_timestamp = []
        for day in range(quotient):
            start_time_temp = Time.to_unix_timestamp(t_start) + int(day * event_duration_nano_seconds)
            event_source_and_timestamp += PmDbRequest.find_events(source, system, className, t_start=start_time_temp,
                                                                  duration=[(event_duration_nano_seconds, 'ns')])
        if remainder > 0:
            start_time_temp = Time.to_unix_timestamp(t_end) - remainder
            event_source_and_timestamp += PmDbRequest.find_events(source, system, className, t_start=start_time_temp,
                                                                  duration=[(remainder, 'ns')])
        return event_source_and_timestamp

    @staticmethod
    def query_context(t_start, t_end, system, className, source, parameters):
        duration = t_end - t_start
        response = PmDbRequest.get_response("pmdata", False, True, pm_rest_api_path="http://pm-api-pro/v2/",
                                            system=system, className=className, source=source,
                                            fromTimestampInNanos=t_start, durationInNanos=duration)
        parameter_to_value = {}

        if response == '':
            parameter_to_value = {parameter: np.nan for parameter in parameters}
            return pd.DataFrame(parameter_to_value, index=[0])

        if system in ['BLM', 'LDBS']:
            for entry in response['content'][0]['namesAndValues']:
                for parameter in parameters:
                    if entry['name'] == parameter:
                        parameter_to_value[parameter] = entry['value']

        elif system == 'LBDS':
            for entry in response['content'][0]['namesAndValues']:
                for parameter in parameters:
                    if entry['name'] == parameter:
                        parameter_to_value[parameter] = max(entry['value'])

            # check for signal validity
            for entry in response['content'][0]['namesAndValues']:
                if (entry['name'] == 'dataValid') and (entry['value'] == 'False'):
                    parameter_to_value = {parameter: 0.0 for parameter in parameters}

        elif system == 'LHC':
            parameter_list = []
            for entry in response['content'][0]['namesAndValues']:
                if entry['name'] == 'registerNames':
                    parameter_list = entry['value']

            for entry in response['content'][0]['namesAndValues']:
                if entry['name'] == 'registerValues':
                    for parameter in parameters:
                        index = parameter_list.index(parameter)
                        parameter_to_value[parameter] = entry['value'][index]

        timestamp = response['content'][0]['rawDataEntry']['pmEventStamp']

        if all([not isinstance(value, (list, tuple)) for value in parameter_to_value.values()]):
            return pd.DataFrame(parameter_to_value, index=[timestamp])
        else:
            df = pd.DataFrame(parameter_to_value)
            df['timestamp'] = timestamp
            return df

    @staticmethod
    def get_blm_signal(t_start, t_end, system, className, source, signal):
        duration = t_end - t_start

        response = PmDbRequest.get_response("pmdata", False, True, pm_rest_api_path="http://pm-api-pro/v2/",
                                                system=system, className=className, source=source,
                                                fromTimestampInNanos=t_start, durationInNanos=duration)
        blm_lst = list(PmDbRequest.query_context(t_start=t_start, t_end=t_end, system=system, className=className,
                                            source=source, parameters=["blmNames"])['blmNames'].values)
        blm_to_signal = {}
        for entry in response['content'][0]['namesAndValues']:
            if entry['name'] == signal:
                for blm_lst_el in blm_lst:
                    index = blm_lst.index(blm_lst_el)
                    blm_to_signal[blm_lst_el] = entry['value'][index]

        return pd.DataFrame(blm_to_signal)

==> lhcsmapi-1.3.300.tar.gz: lhcsmapi-1.3.300/lhcsmapi/dbsignal/post_mortem/PmDbRequest.py <==
import requests
import json
import warnings
import pandas as pd
import numpy as np
import functools
from lhcsmapi.Time import Time


class PmDbRequest(object):
    """PmDbRequest class encapsulates PM database queries in order to simplify the query creation as well as browsing of
    PM events for a given period.
    """

    _request_types = ["pmevent, pmdata", "pmdata/signal"]
    _keyword_to_value_type = {"system": str, "className": str, "source": str, "signal": str,
                              "fromTimestampInNanos": int, "timestampInNanos": int, "durationInNanos": int}

    @staticmethod
    def generate_request(request_type, is_header=False, pm_rest_api_path ="http://pm-api-pro/v2/", **kwargs):
        """**Generates a PostMortem request**

            Method generates a HTTP REST Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration.

            :param request_type: type of PM request (supported: pmdata, pmdata/signal, pmevent)
            :type request_type: string
            :param is_header: flag indicating whether (if True) or not (if False) PM header should be requested
            :type is_header: boolean
            :param kwargs: dictionary of (keyword: argument) pairs which parametrize a db API query
            :type kwargs: dictionary
            :return: string with a PM HTTP request

            - Example::

                >>> from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
                >>> source = "*"
                >>> system = "QPS"
                >>> className = "DQAMCNMB_PMHSU"
                >>> selectedDayTimeStamp = 1426201200000000000
                >>> eventDurationSeconds = 86400000000000
                >>> PmDbRequest.generate_request("pmdata", False, source=source, system=system, className=className, fromTimestampInNanos=selectedDayTimeStamp, duration=eventDurationSeconds) # doctest: +SKIP
                http://pm-api-pro/v2/pmdata/within/duration?source=*&system=QPS&className=DQAMCNMB_PMHSU&fromTimestampInNanos=1426201200000000000&durationInNanos=86400000000000

            - Expected Fail Response::

                >>> from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
                >>> source = "*"
                >>> system = "QPS"
                >>> className = "DQAMCNMB_PMHSU"
                >>> selectedDayTimeStamp = 1426201200000000000
                >>> eventDurationSeconds = 86400000000000
                >>> PmDbRequest.generate_request("pmdata", False, source=source, system=system, className=className, fromTimestampInNanos=selectedDayTimeStamp, duration=eventDurationSeconds) # doctest: +SKIP
                Traceback (most recent call last):
                ...
                ValueError: Some keywords: {'duration'} not in dict_keys(['system', 'className', 'source', 'signal', 'fromTimestampInNanos', 'timestampInNanos', 'durationInNanos'])!

        """

        # check if keys are present
        key_difference = kwargs.keys() - PmDbRequest._keyword_to_value_type.keys()
        if len(key_difference) > 0:
            raise ValueError("Some keywords: {} not in {}!".format(key_difference, PmDbRequest._keyword_to_value_type.keys()))

        # check if value types are correct

        # for time setting check if it is either timestampInNanos and not (fromTimestampInNanos or durationInNanos)
        if "timestampInNanos" in kwargs and any(type in ["fromTimestampInNanos", "durationInNanos"] for type in kwargs):
            raise ValueError("If timestampInNanos is present, then both fromTimestampInNanos and "
                             "durationInNanos have to be absent and vice versa!")

        # concatenate keys and values for a query
        keywords_and_argument_concat = '&'.join(['%s=%s' % (key, value) for (key, value) in kwargs.items()])

        if not any(type in request_type for type in request_type):
            raise ValueError("PM request type {} not supported.".format(request_type))

        request = "{}{}".format(pm_rest_api_path, request_type)

        if is_header:
            request = request + "/header"

        if "durationInNanos" in kwargs:
            return "{}/within/duration?{}".format(request, keywords_and_argument_concat)
        else:
            return "{}?{}".format(request, keywords_and_argument_concat)

    @staticmethod
    @functools.lru_cache(maxsize=16, typed=False)
    def get_response(request_type, is_header=False, display_warning = False, pm_rest_api_path ="http://pm-api-pro/v2/", **kwargs):
        """**Generates a PM request**

            Method returns content of an REST HTTP Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration.

            :param request_type: type of PM request (supported: pmdata, pmdata/signal, pmevent)
            :type request_type: string
            :param is_header: flag indicating whether (if True) or not (if False) PM header should be requested
            :type is_header: boolean
            :param kwargs: dictionary of (keyword: argument) pairs which parametrize a db API query
            :type kwargs: dictionary
            :return: string with a PM HTTP request

            - Example::

                >>> from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
                >>> source = "*"
                >>> system = "QPS"
                >>> className = "DQAMCNMB_PMHSU"
                >>> selectedDayTimeStamp = 1426201200000000000
                >>> eventDurationSeconds = 86400000000000
                >>> PmRequest.getResponse("pmdata", False, source=source, system=system, className=className, fromTimestampInNanos=selectedDayTimeStamp, durationInNanos=eventDurationSeconds) # doctest: +SKIP
                {'content': [{'valueDescriptor': {'attributes': {'QUENCHTIME': {'description': {'signal': 'description',
                ...

            - Expected Fail Response::

                >>> from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
                >>> source = "*"
                >>> system = "QPS"
                >>> className = "DQAMCNMB_PMHSU"
                >>> selectedDayTimeStamp = 1426201200000000000
                >>> eventDurationSeconds = 86400000000000
                >>> PmDbRequest.getResponse("pmdata", False, source=source, system=system, className=className, fromTimestampInNanos=selectedDayTimeStamp, duration=eventDurationSeconds) # doctest: +SKIP
                Traceback (most recent call last):
                ...
                ValueError: Some keywords: {'duration'} not in dict_keys(['system', 'className', 'source', 'signal', 'fromTimestampInNanos', 'timestampInNanos', 'durationInNanos'])!

                >>> from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
                >>> source = "*"
                >>> system = "QPS"
                >>> className = "DQAMCNMB_PMHSU"
                >>> selectedDayTimeStamp = 1426201200000000000
                >>> eventDurationSeconds = 86400
                >>> PmDbRequest.getResponse("pmdata", False, source=source, system=system, className=className, fromTimestampInNanos=selectedDayTimeStamp, durationInNanos=eventDurationSeconds) # doctest: +SKIP
                Traceback (most recent call last):
                ...
                UserWarning: Querying Post Mortem failed using the following query: http://pm-api-pro/v2/pmdata/within/duration?source=*&system=QPS&className=DQAMCNMB_PMHSU&fromTimestampInNanos=1426201200000000001&durationInNanos=86400

        """

        request = PmDbRequest.generate_request(request_type, is_header, pm_rest_api_path, **kwargs)
        response = requests.get(request)

        try:
            return json.loads(response.text)
        except:
            warning = """Querying Post Mortem failed using the following query: {}""".format(request)
            if display_warning:
                warnings.warn(warning)
            return ''

    @staticmethod
    def find_events(source, system, className, t_start, t_end=None, duration=None,
                    pm_rest_api_path="http://pm-api-pro/v2/", display_warning=False):
        """**Finds PM events for a given source, system, className as well as start and end time**

            Method returns content of an REST HTTP Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration. Three operation modes are possible:

            - if endTimeData is not None:

            (t_start, t_end) = (startTimeData, endTimeData)

            - elif len(durationData) is 1:

            (t_start, t_end) = (startTimeData, startTimeData+durationData[0])

            - elif len(durationData) is 2:

            (t_start, t_end) = (startTimeData-durationData[0], startTimeData+durationData[1])

            :param source: PM event source
            :type source: string
            :param system: PM event system
            :type system: string
            :param className: PM event class signal
            :type className: string
            :param t_start: input date and time
            :type t_start: int unix timestamp
            :param t_end: unit of the input (None if duration defined)
            :type t_end: int unix timestamp
            :param duration: time zone (None if t_end defined)
            :type duration: list of (time, unit) tuples
            :return: list of (source, timestamp) pairs

            - Example::

                >>> from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
                >>> source = "*"
                >>> system = "QPS"
                >>> className = "DQAMCNMB_PMHSU"
                >>> selectedDayTimeStamp = 1426201200000000000
                >>> eventDurationSeconds = 24*60*60

                >>> PmDbRequest.find_events(source, system, className, eventTime=selectedDayTimeStamp, duration=[(eventDurationSeconds, 's')]) # doctest: +SKIP
                [('B20L5', 1426220469491000000),
                ...

            - Expected Fail Response::

                >>> from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
                >>> source = "*"
                >>> system = "QPS"
                >>> className = "DQAMCNMB_PMHSU"
                >>> selectedDayTimeStamp = 1426201200000000000
                >>> eventDurationSeconds = 1
                >>> PmDbRequest.finde_vents(source, system, className, eventTime=selectedDayTimeStamp, duration=[(eventDurationSeconds, 's')]) # doctest: +SKIP
                UserWarning: Querying Post Mortem failed using the following query:
                                http://pm-api-pro/v2/pmdata/header/within/duration?system=QPS&className=DQAMCNMB_PMHSU&source=*&fromTimestampInNanos=1426201200000000000&durationInNanos=1000000000 returned no data so an empty string is being returned

        """

        from_timestamp_in_nanos, end_time_in_nanos = Time.get_query_period_in_unix_time(t_start, t_end, duration)
        MAX_DURATION = int(24 * 60 * 60 * 1e9)
        duration_in_nanos = end_time_in_nanos - from_timestamp_in_nanos
        duration_in_nanos = min(duration_in_nanos, MAX_DURATION)

        # Fetch PM event data
        pm_headers = PmDbRequest.get_response("pmdata", True, display_warning, pm_rest_api_path, system=system,
                                              className=className, source=source,
                                              fromTimestampInNanos=from_timestamp_in_nanos, durationInNanos=duration_in_nanos)

        if "v2" in pm_rest_api_path:
            SOURCE_KEYWORD = "source"
        elif "v3" in pm_rest_api_path:
            SOURCE_KEYWORD = "sourceName"
        else:
            raise ValueError('Wrong pm_rest_api_path: %s' % pm_rest_api_path)

        event_list = []
        for pm_header in pm_headers:
            if 'DT' not in pm_header[SOURCE_KEYWORD]:
                event_list.append((pm_header[SOURCE_KEYWORD], pm_header['timestamp']))

            # if 'DT' in 'source', then rubbish data injected into official PM data

        return event_list

    @staticmethod
    def find_events_between_dates(t_start, t_end, className, source, system):
        quotient, remainder = Time.modulo_day_in_unix_timestamp(t_start, t_end)
        event_duration_nano_seconds = int(24 * 60 * 60 * 1e9)
        event_source_and_timestamp = []
        for day in range(quotient):
            start_time_temp = Time.to_unix_timestamp(t_start) + int(day * event_duration_nano_seconds)
            event_source_and_timestamp += PmDbRequest.find_events(source, system, className, t_start=start_time_temp,
                                                                  duration=[(event_duration_nano_seconds, 'ns')])
        if remainder > 0:
            start_time_temp = Time.to_unix_timestamp(t_end) - remainder
            event_source_and_timestamp += PmDbRequest.find_events(source, system, className, t_start=start_time_temp,
                                                                  duration=[(remainder, 'ns')])
        return event_source_and_timestamp

    @staticmethod
    def query_context(t_start, t_end, system, className, source, parameters):
        duration = t_end - t_start
        response = PmDbRequest.get_response("pmdata", False, True, pm_rest_api_path="http://pm-api-pro/v2/",
                                            system=system, className=className, source=source,
                                            fromTimestampInNanos=t_start, durationInNanos=duration)
        parameter_to_value = {}

        if response == '':
            parameter_to_value = {parameter: np.nan for parameter in parameters}
            return pd.DataFrame(parameter_to_value, index=[0])

        if system in ['BLM', 'LDBS']:
            for entry in response['content'][0]['namesAndValues']:
                for parameter in parameters:
                    if entry['name'] == parameter:
                        parameter_to_value[parameter] = entry['value']

        elif system == 'LBDS':
            for entry in response['content'][0]['namesAndValues']:
                for parameter in parameters:
                    if entry['name'] == parameter:
                        parameter_to_value[parameter] = max(entry['value'])

            # check for signal validity
            for entry in response['content'][0]['namesAndValues']:
                if (entry['name'] == 'dataValid') and (entry['value'] == 'False'):
                    parameter_to_value = {parameter: 0.0 for parameter in parameters}

        elif system == 'LHC':
            parameter_list = []
            for entry in response['content'][0]['namesAndValues']:
                if entry['name'] == 'registerNames':
                    parameter_list = entry['value']

            for entry in response['content'][0]['namesAndValues']:
                if entry['name'] == 'registerValues':
                    for parameter in parameters:
                        index = parameter_list.index(parameter)
                        parameter_to_value[parameter] = entry['value'][index]

        timestamp = response['content'][0]['rawDataEntry']['pmEventStamp']

        if all([not isinstance(value, (list, tuple)) for value in parameter_to_value.values()]):
            return pd.DataFrame(parameter_to_value, index=[timestamp])
        else:
            df = pd.DataFrame(parameter_to_value)
            df['timestamp'] = timestamp
            return df

    @staticmethod
    def get_blm_signal(t_start, t_end, system, className, source, signal):
        duration = t_end - t_start

        response = PmDbRequest.get_response("pmdata", False, True, pm_rest_api_path="http://pm-api-pro/v2/",
                                                system=system, className=className, source=source,
                                                fromTimestampInNanos=t_start, durationInNanos=duration)
        blm_lst = list(PmDbRequest.query_context(t_start=t_start, t_end=t_end, system=system, className=className,
                                            source=source, parameters=["blmNames"])['blmNames'].values)
        blm_to_signal = {}
        for entry in response['content'][0]['namesAndValues']:
            if entry['name'] == signal:
                for blm_lst_el in blm_lst:
                    index = blm_lst.index(blm_lst_el)
                    blm_to_signal[blm_lst_el] = entry['value'][index]

        return pd.DataFrame(blm_to_signal)
